/// <reference types="node" />
import NodeFetch from 'node-fetch';
import Window from './Window';
/**
 * Handles the Window.
 */
export default class AsyncWindow extends Window {
    private async;
    /**
     * Constructor.
     */
    constructor();
    /**
     * Sets a timer which executes a function once the timer expires.
     *
     * @param {function} callback Function to be executed.
     * @param {number} [delay] Delay in ms.
     * @return {NodeJS.Timeout} Timeout ID.
     */
    setTimeout(callback: () => void, delay?: number): NodeJS.Timeout;
    /**
     * Cancels a timeout previously established by calling setTimeout().
     *
     * @param {NodeJS.Timeout} id ID of the timeout.
     */
    clearTimeout(id: NodeJS.Timeout): void;
    /**
     * Calls a function with a fixed time delay between each call.
     *
     * @param {function} callback Function to be executed.
     * @param {number} [delay] Delay in ms.
     * @return {NodeJS.Timeout} Interval ID.
     */
    setInterval(callback: () => void, delay?: number): NodeJS.Timeout;
    /**
     * Cancels a timed repeating action which was previously established by a call to setInterval().
     *
     * @param {NodeJS.Timeout} id ID of the interval.
     */
    clearInterval(id: NodeJS.Timeout): void;
    /**
     * Provides a global fetch() method that provides an easy, logical way to fetch resources asynchronously across the network.
     *
     * @param {string} url URL to resource.
     * @param {object} [options] Options.
     * @return {Promise<NodeFetch.Response>} Promise.
     */
    fetch(url: string, options: object): Promise<NodeFetch.Response>;
    /**
     * Returns a promise that is fulfilled when async tasks are complete.
     * This method is not part of the HTML standard.
     *
     * @return {Promise<void>} Promise.
     */
    whenAsyncComplete(): Promise<void>;
    /**
     * Disposes the window.
     */
    dispose(): void;
}
