"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Handles events.
 */
var EventTarget = /** @class */ (function () {
    function EventTarget() {
        this.listeners = {};
    }
    /**
     * Adds an event listener.
     *
     * @param {string} type Event type.
     * @param {function} listener Listener.
     */
    EventTarget.prototype.addEventListener = function (type, listener) {
        this.listeners[type] = this.listeners[type] || [];
        this.listeners[type].push(listener);
    };
    /**
     * Adds an event listener.
     *
     * @param {string} type Event type.
     * @param {function} listener Listener.
     */
    EventTarget.prototype.removeEventListener = function (type, listener) {
        if (this.listeners[type]) {
            var index = this.listeners[type].indexOf(listener);
            if (index !== -1) {
                this.listeners[type].splice(index);
            }
        }
    };
    /**
     * Dispatches an event.
     *
     * @param {Event} event Event.
     * @return {boolean} The return value is false if event is cancelable and at least one of the event handlers which handled this event called Event.preventDefault()
     */
    EventTarget.prototype.dispatchEvent = function (event) {
        var onEventName = 'on' + event.type.toLowerCase();
        var defaultPrevented = false;
        if (typeof this[onEventName] === 'function') {
            this[onEventName].call(this, event);
        }
        if (this.listeners[event.type]) {
            for (var _i = 0, _a = this.listeners[event.type]; _i < _a.length; _i++) {
                var listener = _a[_i];
                listener(event);
                if (event.cancelable && event.defaultPrevented) {
                    defaultPrevented = true;
                }
                if (event.immediatePropagationStopped) {
                    return !defaultPrevented;
                }
            }
        }
        return !defaultPrevented;
    };
    return EventTarget;
}());
exports.default = EventTarget;
//# sourceMappingURL=EventTarget.js.map