import Element from './Element';
import TextNode from './TextNode';
import CommentNode from './CommentNode';
import Window from '../Window';
import Node from './Node';
import NodeType from './NodeType';
import TreeWalker from '../tree-walker/TreeWalker';
import DocumentFragment from './DocumentFragment';
import Event from '../event/Event';
import DOMImplementation from '../implementation/DOMImplementation';
/**
 * Document.
 */
export default class Document extends DocumentFragment {
    documentElement: Element;
    body: Element;
    head: Element;
    defaultView: Window;
    nodeType: NodeType;
    protected elementsById: {
        [k: string]: Element[];
    };
    protected _isConnected: boolean;
    implementation: DOMImplementation;
    /**
     * Creates an instance of Document.
     *
     * @param {Window} window Window instance.
     */
    constructor(window: Window);
    /**
     * Node name.
     *
     * @return {string} Node name.
     */
    readonly nodeName: string;
    /**
     * Replaces the document HTML with new HTML.
     *
     * @param {string} html HTML.
     */
    write(html: string): void;
    /**
     * Opens the document.
     */
    open(): void;
    /**
     * Closes the document.
     */
    close(): void;
    /**
     * Creates an element.
     *
     * @param  {string} tagName Tag name.
     * @return {Element} Element.
     */
    createElement(tagName: string): Element;
    /**
     * Creates a text node.
     *
     * @param  {string} data Text data.
     * @return {TextNode} Text node.
     */
    createTextNode(data: string): TextNode;
    /**
     * Creates a comment node.
     *
     * @param  {string} data Text data.
     * @return {TextNode} Text node.
     */
    createComment(data: string): CommentNode;
    /**
     * Creates a document fragment.
     *
     * @return {DocumentFragment} Document fragment.
     */
    createDocumentFragment(): DocumentFragment;
    /**
     * Creates a Tree Walker.
     *
     * @param {Node} root Root.
     * @param {number} [whatToShow] What to show.
     * @param {(node: Node) => number} [filter] Filter.
     */
    createTreeWalker(root: Node, whatToShow?: number, filter?: (node: Node) => number): TreeWalker;
    /**
     * Creates an event.
     *
     * @legacy
     * @return {Event} Event.
     */
    createEvent(): Event;
    /**
     * Imports a node.
     *
     * @param {Node} node Node to import.
     * @param {Node} Imported node.
     */
    importNode(node: Node): Node;
    /**
     * Returns the element class for a tag name.
     *
     * @param {string} tagName Tag name.
     * @return {typeof Element} Element class.
     */
    private getElementClass;
}
