"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var NodeType_1 = __importDefault(require("./NodeType"));
var Document_1 = __importDefault(require("./Document"));
var ClassList_1 = __importDefault(require("../html-element/ClassList"));
var EventTarget_1 = __importDefault(require("../event/EventTarget"));
var MutationRecord_1 = __importDefault(require("../mutation-observer/MutationRecord"));
var MutationType_1 = __importDefault(require("../mutation-observer/MutationType"));
var ASCII = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'.split('');
/**
 * Node
 */
var Node = /** @class */ (function (_super) {
    __extends(Node, _super);
    /**
     * Constructor.
     */
    function Node() {
        var _this = _super.call(this) || this;
        _this.ownerDocument = null;
        _this.parentNode = null;
        _this.childNodes = [];
        // Protected properties
        _this._isConnected = false;
        // Custom Properties (not part of HTML standard)
        _this.observers = [];
        _this.ownerDocument = _this.constructor.ownerDocument;
        return _this;
    }
    Object.defineProperty(Node.prototype, "isConnected", {
        /**
         * "true" if connected to DOM.
         *
         * @return {boolean} "true" if connected.
         */
        get: function () {
            return this._isConnected;
        },
        /**
         * Sets the connected state.
         *
         * @param {boolean} isConnected "true" if connected.
         */
        set: function (isConnected) {
            if (this._isConnected !== isConnected) {
                this._isConnected = isConnected;
                // eslint-disable-next-line
                if (this.shadowRoot) {
                    // eslint-disable-next-line
                    this.shadowRoot.isConnected = isConnected;
                }
                else {
                    for (var _i = 0, _a = this.childNodes; _i < _a.length; _i++) {
                        var child = _a[_i];
                        child.isConnected = isConnected;
                    }
                }
                if (isConnected && this.connectedCallback) {
                    this.connectedCallback();
                }
                else if (!isConnected && this.disconnectedCallback) {
                    this.disconnectedCallback();
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Node.prototype, "nodeValue", {
        /**
         * Node value.
         *
         * @return {string} Node value.
         */
        get: function () {
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Node.prototype, "nodeName", {
        /**
         * Node name.
         *
         * @return {string} Node name.
         */
        get: function () {
            return '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Node.prototype, "previousSibling", {
        /**
         * Previous sibling.
         *
         * @return {Node} Node.
         */
        get: function () {
            if (this.parentNode) {
                var index = this.parentNode.childNodes.indexOf(this);
                if (index > 0) {
                    return this.parentNode.childNodes[index - 1];
                }
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Node.prototype, "nextSibling", {
        /**
         * Next sibling.
         *
         * @return {Node} Node.
         */
        get: function () {
            if (this.parentNode) {
                var index = this.parentNode.childNodes.indexOf(this);
                if (index > -1 && index + 1 < this.parentNode.childNodes.length) {
                    return this.parentNode.childNodes[index + 1];
                }
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Node.prototype, "firstChild", {
        /**
         * First child.
         *
         * @return {Node} Node.
         */
        get: function () {
            if (this.childNodes.length > 0) {
                return this.childNodes[0];
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Node.prototype, "lastChild", {
        /**
         * Last child.
         *
         * @return {Node} Node.
         */
        get: function () {
            if (this.childNodes.length > 0) {
                return this.childNodes[this.childNodes.length - 1];
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Returns "true" if the node has attributes.
     *
     * @return {boolean} "true" if the node has attributes.
     */
    Node.prototype.hasAttributes = function () {
        return false;
    };
    /**
     * Returns a unique ID.
     *
     * @return {string} ID.
     */
    Node.prototype.getUniqueID = function () {
        if (this.parentNode) {
            var childNodeIndex = this.parentNode.childNodes.indexOf(this);
            var id = ASCII[childNodeIndex] !== undefined ? ASCII[childNodeIndex] : childNodeIndex !== -1 ? childNodeIndex : '';
            return this.parentNode.getUniqueID() + id;
        }
        return 'a';
    };
    /**
     * Clones a node.
     *
     * @return {Node} Cloned node.
     */
    Node.prototype.cloneNode = function () {
        var clone = new this.constructor();
        for (var _i = 0, _a = Object.keys(this); _i < _a.length; _i++) {
            var key = _a[_i];
            if (key === 'childNodes') {
                for (var _b = 0, _c = this[key]; _b < _c.length; _b++) {
                    var childNode = _c[_b];
                    var childClone = childNode.cloneNode();
                    childClone.parentNode = clone;
                    clone.childNodes.push(childClone);
                }
            }
            else if (key !== 'parentNode' && key !== 'ownerDocument' && this[key] instanceof Node) {
                clone[key] = this[key].cloneNode();
                clone[key].parentNode = clone;
            }
            else if (key === 'classList') {
                // eslint-disable-next-line
                clone[key] = new ClassList_1.default(clone);
            }
            else if (this[key] !== null &&
                typeof this[key] === 'object' &&
                !(this[key] instanceof Document_1.default) &&
                !(this[key] instanceof Node)) {
                clone[key] = Object.assign({}, this[key]);
            }
            else {
                clone[key] = this[key];
            }
        }
        return clone;
    };
    /**
     * Append a child node to childNodes.
     *
     * @param  {Node} node Node to append.
     * @return {Node} Appended node.
     */
    Node.prototype.appendChild = function (node) {
        if (node === this) {
            throw new Error('Not possible to append self as child self.');
        }
        if (node.nodeType === NodeType_1.default.DOCUMENT_FRAGMENT_NODE) {
            for (var _i = 0, _a = node.childNodes.slice(); _i < _a.length; _i++) {
                var child = _a[_i];
                this.appendChild(child);
            }
            return node;
        }
        if (node.parentNode) {
            var index = node.parentNode.childNodes.indexOf(node);
            if (index !== -1) {
                node.parentNode.childNodes.splice(index, 1);
            }
        }
        this.childNodes.push(node);
        node.parentNode = this;
        node.isConnected = this.isConnected;
        // MutationObserver
        if (this.observers.length > 0) {
            var record = new MutationRecord_1.default();
            record.type = MutationType_1.default.childList;
            record.addedNodes = [node];
            for (var _b = 0, _c = this.observers; _b < _c.length; _b++) {
                var observer = _c[_b];
                if (observer.options.subtree) {
                    node.observe(observer);
                }
                if (observer.options.childList) {
                    observer.callback([record]);
                }
            }
        }
        return node;
    };
    /**
     * Remove Child element from childNodes array.
     *
     * @param {Node} node Node to remove
     */
    Node.prototype.removeChild = function (node) {
        var index = this.childNodes.indexOf(node);
        if (index === -1) {
            throw new Error('Failed to remove node. Node is not child of parent.');
        }
        this.childNodes.splice(index, 1);
        node.parentNode = null;
        node.isConnected = false;
        // MutationObserver
        if (this.observers.length > 0) {
            var record = new MutationRecord_1.default();
            record.type = MutationType_1.default.childList;
            record.removedNodes = [node];
            for (var _i = 0, _a = this.observers; _i < _a.length; _i++) {
                var observer = _a[_i];
                node.unobserve(observer);
                if (observer.options.childList) {
                    observer.callback([record]);
                }
            }
        }
    };
    /**
     * Inserts a node before another.
     *
     * @param {Node} newNode Node to insert.
     * @param {Node} referenceNode Node to insert before.
     * @return {Node} Inserted node.
     */
    Node.prototype.insertBefore = function (newNode, referenceNode) {
        if (newNode.nodeType === NodeType_1.default.DOCUMENT_FRAGMENT_NODE) {
            for (var _i = 0, _a = newNode.childNodes.slice(); _i < _a.length; _i++) {
                var child = _a[_i];
                this.insertBefore(child, referenceNode);
            }
            return newNode;
        }
        if (referenceNode === null) {
            return this.appendChild(newNode);
        }
        var index = this.childNodes.indexOf(referenceNode);
        if (index === -1) {
            throw new Error('Failed to insert node. Reference node is not child of parent.');
        }
        if (newNode.parentNode) {
            var index_1 = newNode.parentNode.childNodes.indexOf(newNode);
            if (index_1 !== -1) {
                newNode.parentNode.childNodes.splice(index_1, 1);
            }
        }
        this.childNodes.splice(index, 0, newNode);
        newNode.parentNode = this;
        newNode.isConnected = this.isConnected;
        // MutationObserver
        if (this.observers.length > 0) {
            var record = new MutationRecord_1.default();
            record.type = MutationType_1.default.childList;
            record.addedNodes = [newNode];
            for (var _b = 0, _c = this.observers; _b < _c.length; _b++) {
                var observer = _c[_b];
                if (observer.options.subtree) {
                    newNode.observe(observer);
                }
                if (observer.options.childList) {
                    observer.callback([record]);
                }
            }
        }
        return newNode;
    };
    /**
     * Replaces a node with another.
     *
     * @param {Node} newChild New child.
     * @param {Node} oldChild Old child.
     * @return {Node} Replaced node.
     */
    Node.prototype.replaceChild = function (newChild, oldChild) {
        this.insertBefore(newChild, oldChild);
        this.removeChild(oldChild);
        return oldChild;
    };
    /**
     * Observeres the node.
     * Used by MutationObserver, but it is not part of the HTML standard.
     *
     * @param {MutationObserverListener} listener Listener.
     */
    Node.prototype.observe = function (listener) {
        this.observers.push(listener);
        if (listener.options.subtree) {
            for (var _i = 0, _a = this.childNodes; _i < _a.length; _i++) {
                var node = _a[_i];
                node.observe(listener);
            }
        }
    };
    /**
     * Stops observing the node.
     * Used by MutationObserver, but it is not part of the HTML standard.
     *
     * @param {MutationObserverListener} listener Listener.
     */
    Node.prototype.unobserve = function (listener) {
        var index = this.observers.indexOf(listener);
        if (index !== -1) {
            this.observers.splice(index, 1);
        }
        if (listener.options.subtree) {
            for (var _i = 0, _a = this.childNodes; _i < _a.length; _i++) {
                var node = _a[_i];
                node.unobserve(listener);
            }
        }
    };
    // Public properties
    Node.ownerDocument = null;
    return Node;
}(EventTarget_1.default));
exports.default = Node;
// Adds Note types to the Node class (part of the HTML standard)
for (var _i = 0, _a = Object.keys(NodeType_1.default); _i < _a.length; _i++) {
    var key = _a[_i];
    Node[key] = NodeType_1.default[key];
}
//# sourceMappingURL=Node.js.map