import Node from '../nodes/basic-types/Node';
/**
 * The TreeWalker object represents the nodes of a document subtree and a position within them.
 */
export default class TreeWalker {
    root: Node;
    whatToShow: number;
    filter: (node: Node) => number;
    currentNode: Node;
    /**
     * Constructor.
     *
     * @param {Node} root Root.
     * @param {number} [whatToShow] What to show.
     * @param {function} [filter] Filter.
     */
    constructor(root: Node, whatToShow?: number, filter?: (node: Node) => number);
    /**
     * Moves the current Node to the first visible ancestor node in the document order, and returns the found node. It also moves the current node to this one. If no such node exists, or if it is before that the root node defined at the object construction, returns null and the current node is not changed.
     *
     * @return {Node} Current node.
     */
    parentNode(): Node;
    /**
     * Moves the current Node to the first visible child of the current node, and returns the found child. It also moves the current node to this child. If no such child exists, returns null and the current node is not changed.
     *
     * @return {Node} Current node.
     */
    firstChild(): Node;
    /**
     * Moves the current Node to the last visible child of the current node, and returns the found child. It also moves the current node to this child. If no such child exists, null is returned and the current node is not changed.
     *
     * @return {Node} Current node.
     */
    lastChild(): Node;
    /**
     * Moves the current Node to its previous sibling, if any, and returns the found sibling. If there is no such node, return null and the current node is not changed.
     *
     * @return {Node} Current node.
     */
    previousSibling(): Node;
    /**
     * Moves the current Node to its next sibling, if any, and returns the found sibling. If there is no such node, null is returned and the current node is not changed.
     *
     * @return {Node} Current node.
     */
    nextSibling(): Node;
    /**
     * Moves the current Node to the next visible node in the document order.
     *
     * @return {Node} Current node.
     */
    nextNode(): Node;
    /**
     * Moves the current Node to the previous visible node in the document order, and returns the found node. It also moves the current node to this one. If no such node exists, or if it is before that the root node defined at the object construction, returns null and the current node is not changed.
     *
     * @return {Node} Current node.
     */
    previousNode(): Node;
    /**
     * Filters a node.
     *
     * @param {Node} node Node.
     * @return {boolean} Child nodes.
     */
    private filterNode;
    /**
     * Filters what to show.
     *
     * @param {Node} node Node.
     * @return {boolean} TRUE if the element should be shown.
     */
    private filterWhatToShow;
}
